import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Hashtable;
import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/** This example shows how to establish a connection
 * and send messages to the JMS queue. The classes in this
 * package operate on the same JMS queue. Run the classes together to
 * witness messages being sent and received, and to browse the queue
 * for messages. The class is used to send messages to the queue.
 *
 * @author Copyright (c) 1999-2008 by BEA Systems, Inc. All Rights Reserved.
 */
public class QueueSend
{
  // Defines the JNDI context factory.
  public final static String JNDI_FACTORY="weblogic.jndi.WLInitialContextFactory";

  // Defines the JMS context factory.
  public final static String JMS_FACTORY="sourceConnectionFactory";

  // Defines the queue.
  public final static String QUEUE="remoteQueue";

  public final static boolean TRACE=false;

  private QueueConnectionFactory qconFactory;
  private QueueConnection qcon;
  private QueueSession qsession;
  private QueueSender qsender;
  private Queue queue;
  private TextMessage msg;

  /**
   * Creates all the necessary objects for sending
   * messages to a JMS queue.
   *
   * @param ctx JNDI initial context
   * @param queueName name of queue
   * @exception NamingException if operation cannot be performed
   * @exception JMSException if JMS fails to initialize due to internal error
   */
  public void init(Context ctx, String queueName)
    throws NamingException, JMSException
  {
      trace("Getting QueueConnectionFactory [" + JMS_FACTORY + "]...");
      qconFactory = (QueueConnectionFactory) ctx.lookup(JMS_FACTORY);
      trace("Done\n");

      trace("createQueueConnection()...");
      qcon = qconFactory.createQueueConnection();
      trace("Done\n");

      trace("createQueueSession()...");
      qsession = qcon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
      trace("Done\n");

      trace("Getting Queue [" + QUEUE + "]...");
      queue = (Queue) ctx.lookup(queueName);
      trace("Done\n");

      trace("createSender...");
      qsender = qsession.createSender(queue);
      trace("Done\n");

      msg = qsession.createTextMessage();
      qcon.start();
  }


  /**
   * Loops, taking input and sending JMS message
   * Retry logic included to handle cluster communication
   *
   * @param ctx JNDI initial context
   * @param queueName name of queue
   * @exception NamingException if operation cannot be performed
   * @exception JMSException if JMS fails to initialize due to internal error
   * @exception InterruptedException if internal interrupt
   * @exception IOException if error in user input
   */
  public void doQueueSend(Context ctx, String queueName)
    throws NamingException, JMSException, InterruptedException, IOException
  {
    boolean quitNow = false;
    boolean resendNeeded = false;
    String line=null;
    BufferedReader msgStream = new BufferedReader(new InputStreamReader(System.in));

    do {
      try { 
        init(ctx, QUEUE);
    
        do {
          if (!resendNeeded) {
            System.out.print("Enter message (\"quit\" to quit): \n");
            line = msgStream.readLine();
          }
          if (line != null && line.trim().length() != 0) {
            msg.setText(line);
            resendNeeded=true;

            trace("send()...");
            qsender.send(msg);
            trace("Done\n");

            System.out.println("JMS Message Sent: "+line+"\n");
            resendNeeded=false;
            quitNow = line.equalsIgnoreCase("quit");
          }
        } while (! quitNow);
    
      } 
      catch(Exception e) {
        trace("Exception: " + e.getMessage() + "\n");
        trace("Retrying...\n");
        close();
      }
    } while (! quitNow);
  }


  /**
   * Closes JMS objects.
   * @exception JMSException if JMS fails to close objects due to internal error
   */
  public void close() throws JMSException {
    qsender.close();
    qsession.close();
    qcon.close();
  }


  /** main() method.
  *
  * @param args WebLogic Server URL
  * @exception Exception if operation fails
  */
  public static void main(String[] args) throws Exception {
    if (args.length != 1) {
      System.out.println("Usage: java examples.jms.queue.QueueSend WebLogicURL");
      return;
    }
    try {
      InitialContext ic = getInitialContext(args[0]);

      QueueSend qs = new QueueSend();
      qs.doQueueSend(ic, QUEUE);
      qs.close();

    } catch(JMSException jmse) {
      System.err.println("Exception: "+jmse.getMessage());
    }
  }


  private static InitialContext getInitialContext(String url)
    throws NamingException
  {
    Hashtable<String,String> env = new Hashtable<String,String>();
    env.put(Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY);
    env.put(Context.PROVIDER_URL, url);
    return new InitialContext(env);
  }


  private static void trace(String s)
  {
    if (TRACE) System.out.print(s);
  }
}


