package examples.cluster.ejb.statefulSession;

import java.rmi.RemoteException;
import java.util.Hashtable;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * This class illustrates calling a stateful SessionBean and performing
 * the following exercises:
 * <ul>
 * <li> Create a Trader
 * <li> Buy some shares using the Trader
 * <li> Sell some shares using the Trader
 * <li> Remove the Trader
 * </ul>
 *
 * @author Copyright (c) 2008 by BEA Systems, Inc. All Rights Reserved.
 */
public class Client {
  private static final String JNDI_NAME = "cluster-ejb-stateful-TraderHome";
  private TraderHome home;
  private Trader trader;
  private static String url = "t3://localhost:7003,localhost:7005,localhost:7007";  

  public Client(String url) throws NamingException, CreateException, RemoteException  {
    this.url = url;
    home = lookupHome();
    // Create a Trader
    log("Creating trader\n");
    trader = (Trader) narrow(home.create(), Trader.class);
  }

  /**
   * Run this example from the command line.
   */
  public static void main(String[] args) throws Exception {
    log("\nBeginning statefulSession.Client...\n");
    Client client = null;

    try {
      client = new Client(url);
      client.interaction();
    } catch (NamingException ne) {
      log("Unable to look up the beans home: " + ne.getMessage());
      throw ne;
    } catch (Exception e) {
      log("There was an exception while creating and using the Trader.");
      log("This indicates that there was a problem communicating with the server: "+e);
      throw e;
    }
    log("\nEnd statefulSession.Client...\n");
  }


  /**
   * Process Trades
   */
  public void processTrades() throws CreateException, IOException,
          ProcessingErrorException, RemoteException, RemoveException {
    log("Processing Preordered Trades...");
    String customerName = "Matt";

    // Sell some stock
    String stockName   = "BEAS";
    int numberOfShares = 200;
    log("Selling " + numberOfShares + " of " + stockName);
    TradeResult tr = trader.sell(customerName, stockName, numberOfShares);
    log(tr.toString());

    // Buy some stock
    stockName      = "ORCL";
    numberOfShares = 250;
    log("Buying " + numberOfShares + " of " + stockName);
    tr = trader.buy(customerName, stockName, numberOfShares);
    log(tr.toString());
  }

  /**
   * Command line interaction
   */
  public void interaction() throws IOException, ProcessingErrorException,
          RemoteException, RemoveException {
    BufferedReader msgStream = new BufferedReader(
        new InputStreamReader(System.in));
    String line=null;
    boolean quitNow = false;
    do {
      log("Enter \"bal\", \"trade\" or \"quit\" to quit: ");
      line = msgStream.readLine();
      if (line != null && line.trim().length() != 0) {
        if (line.equalsIgnoreCase("bal")) {
          log("Change in Cash Account: $" + trader.getBalance() );
        }
 
        if (line.equalsIgnoreCase("trade")) {
        try {         
          processTrades();
        } catch (Exception e) {
          log("Exception: " + e.getMessage() );
        }
        }

        quitNow = line.equalsIgnoreCase("quit");
      }
    } while (! quitNow);

    log("Removing trader\n");
    trader.remove();
  }

  /**
   * RMI/IIOP clients should use this narrow function
   */
  private Object narrow(Object ref, Class c) {
    return PortableRemoteObject.narrow(ref, c);
  }

  /**
   * Lookup the EJBs home in the JNDI tree
   */
  private TraderHome lookupHome() throws NamingException {
    // Lookup the beans home using JNDI
    Context ctx = getInitialContext();
    try {
      Object home = ctx.lookup(JNDI_NAME);
      return (TraderHome) narrow(home, TraderHome.class);
    } catch (NamingException ne) {
      log("The client was unable to lookup the EJBHome.  Please make sure ");
      log("that you have deployed the ejb with the JNDI name "+
          JNDI_NAME+" on the WebLogic server at "+url);
      throw ne;
    }
  }

  private Context getInitialContext() throws NamingException {
    try {
      // Get an InitialContext
      Hashtable<String,String> h = new Hashtable<String,String>();
      h.put(Context.INITIAL_CONTEXT_FACTORY,
        "weblogic.jndi.WLInitialContextFactory");
      h.put(Context.PROVIDER_URL, url);
      return new InitialContext(h);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }

  private static void log(String s) { System.out.println(s);  }
}
