import javax.management.*;

import weblogic.diagnostics.watch.*;

import weblogic.diagnostics.watch.JMXWatchNotification;
import javax.management.Notification;
import javax.management.remote.JMXServiceURL;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXConnector;
import javax.naming.Context;
import java.util.Hashtable;
import weblogic.management.mbeanservers.runtime.RuntimeServiceMBean;

/**
 * This class implements a simple JMX listener to dump the WLDF JMX 
 * notifications to stdout, utilizing JMX filters to restrict notifications
 * to be sent only to the type being used in the watch setup.  It registers 
 * with the W&N JMX broadcaster MBean singleton; there is only one JMX 
 * broadcaster MBean for all JMX Watch notifications.
 */
public class JMXWatchNotificationListener implements NotificationListener, Runnable {

  private MBeanServerConnection rmbs = null;
  private String notifName = "LabJMXNotification";
  private int notifCount = 0;
  private String serverName = "AdminServer";
  private String user = "weblogic";
  private String password = "weblogic1";

  private ObjectName watchBroadcasterObjectName = null;
  
  public JMXWatchNotificationListener(String serverName) {
    this.serverName = serverName;
  }

  public void register() throws Exception {
    watchBroadcasterObjectName =
      new ObjectName(
        "com.bea:ServerRuntime=" + this.serverName + ",Name=" +  
        JMXWatchNotification.GLOBAL_JMX_NOTIFICATION_PRODUCER_NAME + 
        ",Type=WLDFWatchJMXNotificationRuntime," +
        "WLDFWatchNotificationRuntime=WatchNotification," +
        "WLDFRuntime=WLDFRuntime"
        );
    rmbs = getRuntimeMBeanServerConnection();
    addNotificationHandler();
  }

  /**
   * Implements the standard JMX notification handler; dumps the received 
   * JMXWatchNotification to stdout.
   */
  public void handleNotification(Notification notif, Object handback) {
    synchronized (this) {
      try {
        if (notif instanceof JMXWatchNotification) {
          WatchNotification wNotif = ((JMXWatchNotification)notif).getExtendedInfo();
          notifCount++;
  
          System.out.println("=========================================================================");
          System.out.println("Notification name: " + notifName + " called. Count= " + notifCount + ".");
  
          System.out.println("Watch severity:         " + wNotif.getWatchSeverityLevel());
          System.out.println("Watch time:             " + wNotif.getWatchTime());
          System.out.println("Watch ServerName:       " + wNotif.getWatchServerName());
          System.out.println("Watch RuleType:         " + wNotif.getWatchRuleType());
          System.out.println("Watch Rule:             " + wNotif.getWatchRule());
          System.out.println("Watch Name:             " + wNotif.getWatchName());
          System.out.println("Watch DomainName:       " + wNotif.getWatchDomainName());
          System.out.println("Watch AlarmType:        " + wNotif.getWatchAlarmType());
          System.out.println("Watch AlarmResetPeriod: " + wNotif.getWatchAlarmResetPeriod());
          System.out.println("=========================================================================");
        }
      } catch (Throwable x) {
        System.out.println("Exception occurred processing JMX watch notification: " + 
                           notifName +"\n" + x);
        x.printStackTrace();
      }
    }
  }    

  private void addNotificationHandler() throws Exception {
    /*
     * The JMX Watch notification listener registers with a Runtime MBean that
     * matches the name of the corresponding watch bean.  Each watch has it's
     * own Runtime MBean instance.
     */
    NotificationFilterSupport filter = new NotificationFilterSupport();
    filter.enableType("wldf.demo.jmxnotification");
    System.out.println("Adding notification handler for: " + 
                        watchBroadcasterObjectName.getCanonicalName());
    rmbs.addNotificationListener(watchBroadcasterObjectName, this, null, null);
    //rmbs.addNotificationListener(watchBroadcasterObjectName, this, filter, null);
  }

  private void removeNotificationHandler(String name, NotificationListener list) throws Exception {
    ObjectName oname = 
      new ObjectName(
          "com.bea:ServerRuntime=" + serverName + ",Name=" +  
          JMXWatchNotification.GLOBAL_JMX_NOTIFICATION_PRODUCER_NAME + 
          ",Type=WLDFWatchJMXNotificationRuntime," +
          "WLDFWatchNotificationRuntime=WatchNotification," +
          "WLDFRuntime=WLDFRuntime"
          );
    System.out.println("Removing notification handler for: " + 
                       oname.getCanonicalName());
    rmbs.removeNotificationListener(oname, list);
  }

  /**
   * Shutdown hook run method, invoked by the shutdown hook by the VM.
   */
  public void run() {
    try {
      System.out.println("VM shutdown, unregistering notification listener");
      removeNotificationHandler(notifName, this);
    } catch (Throwable t) {
      System.out.println("Caught exception in shutdown hook");
      t.printStackTrace();
    }
  }

  public MBeanServerConnection getRuntimeMBeanServerConnection()
    throws Exception {
    
    String JNDI = "/jndi/";

    JMXServiceURL serviceURL;
    serviceURL =
      new JMXServiceURL("t3", "localhost", 7001,
                        JNDI + RuntimeServiceMBean.MBEANSERVER_JNDI_NAME);
    System.out.println("URL=" + serviceURL);

    Hashtable h = new Hashtable();
    h.put(Context.SECURITY_PRINCIPAL,user);
    h.put(Context.SECURITY_CREDENTIALS,password);
    h.put(JMXConnectorFactory.PROTOCOL_PROVIDER_PACKAGES,
          "weblogic.management.remote");
    JMXConnector connector = JMXConnectorFactory.connect(serviceURL,h);
    return connector.getMBeanServerConnection();
  }

  /**
   * Instantiate the listener, add the shutdown hook to the VM runtime, 
   * and go to sleep indefinitely.
   * 
   * @param args
   */
  public static void  main(String[] args) {
    try {
      String serverName = "AdminServer";
      if (args.length > 0)
        serverName = args[0];
      JMXWatchNotificationListener listener = 
          new JMXWatchNotificationListener(serverName);
      System.out.println("Adding shutdown hook");
      Runtime.getRuntime().addShutdownHook(new Thread(listener));
      listener.register();
      // Sleep waiting for notifications
      Thread.sleep(Long.MAX_VALUE);
    } catch (Throwable e) {
      e.printStackTrace();
    } // end of try-catch
    
  } // end of main()
  
}
