from weblogic.descriptor import BeanAlreadyExistsException

execfile('WLDFResource.py')

"""
WLDF Profiles base class

 This provides a common framework for defining WLDF Harvester and W&N Profiles.

 Each profile contains 3 maps
 - A map of arrays of harvested attributes, keyed by the MBean type to harvest from
 - A map of Watch rules, keyed by watch name, where the payload is a map of Watch properties
 - A map of Instrumentation monitors, keyed by the monitor name, and the payload is a map of monitor properties

 The structure of the harvested types map consists of
  { type1 : [ attribute1, attribute2, ... ]
    type2 : [ attribute1, attribute2, ... ]
    ...
  }

 The structure of the Watch rules map consists of
  { watchName1 :  { watchName1-props-map } 
    watchName2 :  { watchName2-props-map } 
    ...
  }

 where each watch rule properties map can consist of the following 
 key/value pairs:

 Key                                         Value
 =================     =================================
 'rule'                                         The watch rule expression
 'ruleType'                                 The watch rule type ('Harvester', 'Log', 'EventData')
 'resetType'                               The alarm type of the watch ('ManualReset', 'AutomaticReset')
 'resetPeriod'                             The alarm reset period of the watch, in millis

The structure of the Montiors map is as follows:

  { monitorName1 :  { monitorName1-props-map } 
    monitorName2 :  { monitorName2-props-map } 
    ...
  }

 where each monitor's properties map can consist of the following 
 key/value pairs:

 Key                                         Value
 =================     =================================
 'props'                                      The monitor properties (provide via 'dyeProps' command-line arg)
 'desc'                                       The monitor description
 'dyeMask'                                 The dye-mask for the monitor, if dye filtering is enabled
'dyeFiltering'                              The dye-filtering enabled flag

Note that there are other attributes on the WLDFInstrumentationMonitorBean; since only
Standard and Delegating monitors can be set in the server scope, only the above 
properties apply.


 Each profile that subclasses this base must set the values of the following 
 attributes:
 - 'attributes'  (Harvested types map)
 - 'watches'     (Watch rule config map)
- 'monitors'  (monitors config)

 A profile does not need to implement this base class; if it does not, it must provide
 the same public interface methods to meet the contract defined and used by 
 the WLDFProfileManager class.
"""
class WLDFProfileBase:
  packagePrefix="weblogic.management.runtime."

  attributes = {}
  watches = {}
  monitors = {}
  __wlsVer = None
  
  def getHarvestedAttributes(self):
    return self.attributes
  
  def getWatchRules(self):
    return self.watches
  
  def getInstrumentationMonitors(self):
    return self.monitors
  
  """
  Used to determine the version of WLS we are running on; this is 
  necessary to avoid using syntax that is not available prior to
  WLS 10.3 when running on earlier versions. 
  """
  def getWLSVersion(self):
    if self.__wlsVer is None:
      try:
        self.__wlsVer = float(version.split()[2])
      except:
        self.__wlsVer = 9.0
    return self.__wlsVer
  
# "Core" profile
#
class WLSCoreProfile(WLDFProfileBase):

  def __init__(self, argsDict=None):
    workManagerType = self.packagePrefix + "WorkManagerRuntimeMBean"
    srtType = self.packagePrefix + "ServerRuntimeMBean"
    clusterType = self.packagePrefix + "ClusterRuntimeMBean"
    threadPoolType = self.packagePrefix + 'ThreadPoolRuntimeMBean'
    jvmType = self.packagePrefix + "JVMRuntimeMBean"
    jrockitType = self.packagePrefix + 'JRockitRuntimeMBean'
    
    self.attributes = {
      workManagerType : [ 'StuckThreadCount' , 'PendingRequests', 'CompletedRequests' ],
      srtType :         [ 'OpenSocketsCurrentCount' , 'SocketsOpenedTotalCount' ],
      clusterType :     [ 'AliveServerCount' ],
      threadPoolType :  [  'CompletedRequestCount', 'PendingUserRequestCount', 'HoggingThreadCount', 
                              'QueueLength', 'ExecuteThreadIdleCount', 'StandbyThreadCount',
                              'ExecuteThreadTotalCount', 'Throughput'],
      jvmType :         [ 'HeapSizeCurrent', 'HeapFreePercent', 'HeapFreeCurrent', 'HeapSizeMax' ],
      jrockitType :     [ 'HeapSizeCurrent', 'HeapFreePercent', 'HeapFreeCurrent', 
                              'HeapSizeMax', 'JvmProcessorLoad', 'AllProcessorsAverageLoad']
    }
    
    print "core args: " + str(argsDict)
    dyeProps = getValue(argsDict, 'dyeProps', 'USER1=weblogic').replace(',','\n')
    print "dye properties: "  + dyeProps
    self.monitors = {
           "DyeInjection"  : { 'props' : dyeProps, 'actions' : None, 'desc' : "DyeInjection monitor",  'dyeMask' : None, 'dyeFiltering' : false }
        }
    
    self.watches = {
      'StuckThreadWatch' : 
        { 'rule' : '${[' + workManagerType + ']//StuckThreadCount} > 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'ServerHealthStateWatch' :
        {'rule' : '${[' + srtType +']//State} IN (\'FAILED\',\'FAILED_RESTARTING\',\'FAILED_NOT_RESTARTABLE\')', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'ClusterServerWatch' :
        {'rule' : '${[' + clusterType +']//AliveServerCount} = 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'HeapWatch' :
        {'rule' : '${[' + jvmType +']//HeapFreePercent} < 10', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'JRockitHeapWatch' :
        {'rule' : '${[' + jrockitType +']//HeapFreePercent} < 10', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'JRockitCPUWatch' :
        {'rule' : '${[' + jrockitType +']//AllProcessorsWatch} > 0.9', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'ServerLogWatch' :
        {'rule' : "SEVERITY IN ('CRITICAL', 'ERROR', 'WARNING')", 'ruleType' : 'Log', 'resetType' : 'AutomaticReset'},
    }
    
    if self.getWLSVersion() >= 10.3:
      slcType = self.packagePrefix + "ServerLifeCycleRuntimeMBean"
      self.watches['ServerLifeCycleWatch'] = {
        'rule' : '${DomainRuntime//[' + slcType +']//State} IN (\'FAILED\',\'FAILED_RESTARTING\',\'FAILED_NOT_RESTARTABLE\')', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'
        }
      self.watches['ClusterHealthWatch'] = {
        'rule' : '${[' + clusterType +']//HealthState.State} != 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'
        }
  
# EJB profile
#
class WLSEJBProfile(WLDFProfileBase):

  def __init__(self):
    
    ejbCacheType = self.packagePrefix + 'EJBCacheRuntimeMBean' 
    ejbCacheAttributes = [  'CacheHitCount', 'CacheMissCount' ]
    
    # EJBLockingRuntimeMBean
    ejbLockType = self.packagePrefix + 'EJBLockingRuntimeMBean' 
    ejbLockAttributes = [  'LockEntriesCurrentCount', 'WaiterCurrentCount' ]

    # MDB 
    mdbType = self.packagePrefix + 'MessageDrivenEJBRuntimeMBean' 
    mdbAttributes = [ ]
    mdbWatchRules = { 
      'MDBHealthWatch' : 
        { 'rule' : '${[' + mdbType + ']//HealthState.State} != 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      }

    # EJBTimerRuntimeMBean 
    ejbTimerType = self.packagePrefix + 'EJBTimerRuntimeMBean' 
    ejbTimerAttributes = [ 'ActiveTimerCount', 'CancelledTimerCount', 'DisabledTimerCount', 'TimeoutCount' ]

    # EJBTransactionRuntimeMBean 
    ejbTxType = self.packagePrefix + 'EJBTransactionRuntimeMBean' 
    ejbTxAttributes = [ 'TransactionsCommittedTotalCount', 'TransactionsRolledBackTotalCount', 'TransactionsTimedOutTotalCount' ]
  
    # QueryCacheRuntimeMBean 
    queryCacheType = self.packagePrefix + 'QueryCacheRuntimeMBean' 
    queryCacheAttributes = [ 'CacheAccessCount', 'CacheHitCount', 'TotalCachedQueriesCount', 'TotalCacheMissCount' ]
  
    # EJBPoolRuntimeMBean 
    ejbPoolType = self.packagePrefix + 'EJBPoolRuntimeMBean' 
    ejbPoolAttributes = [ 'BeansInUseCurrentCount', 'IdleBeansCount', 'PooledBeansCurrentCount', 'WaiterCurrentCount' ]
  
    # KodoDataCacheRuntimeMBean 
    kodoCacheType = self.packagePrefix + 'KodoDataCacheRuntimeMBean' 
    kodoCacheAttributes = [ 'CacheHitRatio', 'TotalCurrentEntries' ]
  
    # KodoQueryCacheRuntimeMBean 
    kodoQueryCacheType = self.packagePrefix + 'KodoQueryCacheRuntimeMBean' 
    kodoQueryCacheAttributes = [ 'CacheHitRatio', 'TotalCurrentEntries' ]
  
    # KodoQueryCompilationCacheRuntimeMBean 
    kodoCompileCacheType = self.packagePrefix + 'KodoQueryCompilationCacheRuntimeMBean' 
    kodoCompileCacheAttributes = [ 'TotalCurrentEntries' ]
    
    self.attributes = {
      ejbCacheType         : ejbCacheAttributes,
      ejbLockType          : ejbLockAttributes,
      mdbType              : mdbAttributes,
      ejbTimerType         : ejbTimerAttributes,
      ejbTxType            : ejbTxAttributes,
      queryCacheType       : queryCacheAttributes,
      ejbPoolType          : ejbPoolAttributes,
      kodoCacheType        : kodoCacheAttributes,
      kodoQueryCacheType   : kodoQueryCacheAttributes, 
      kodoCompileCacheType : kodoCompileCacheAttributes
    }

    if self.getWLSVersion() >= 10.3 :
      self.watches = mdbWatchRules

# JDBC profile
#
class WLSJDBCProfile(WLDFProfileBase):

  def __init__(self):

    jdbcDSType = 'weblogic.management.runtime.JDBCDataSourceRuntimeMBean'
    jdbcDSAttributes = ['ActiveConnectionsAverageCount', 'ActiveConnectionsCurrentCount', 'CurrCapacity', 'LeakedConnectionCount', 'NumAvailable', 
      'NumUnavailable', 'PrepStmtCacheHitCount', 'PrepStmtCacheMissCount', 'ConnectionDelayTime', 'FailuresToReconnectCount', 
      'WaitingForConnectionFailureTotal', 'WaitingForConnectionCurrentCount']

    jdbcDSWatches = {
      'AvailableDSWatch' : { 'rule' : '${[' + jdbcDSType + ']//NumAvailable} = 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'ConnectionDelayWatch' : { 'rule' : '${[' + jdbcDSType + ']//ConnectionDelayTime} > 1000', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'},
      'ConnectFailures' : { 'rule' : '${[' + jdbcDSType + ']//FailuresToReconnectCount} > 0', 'ruleType' : 'Harvester', 'resetType' : 'ManualReset'}
      }
    
    self.attributes = { jdbcDSType : jdbcDSAttributes }
    self.watches = jdbcDSWatches
    
    if self.getWLSVersion() >= 10.3:
      self.watches['AvailableDSWatch'] = { 
        'rule' : '${[' + self.packagePrefix + 'JDBCServiceRuntime]//HealthState.State} != 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset'
        }

# JMS Profile
#
class WLSJMSProfile(WLDFProfileBase):

  def __init__(self):
    
    jmsConnectionType =  self.packagePrefix + "JMSConnectionRuntimeMBean"
    jmsConnectionAttributes = [ 'SessionsCurrentCount' ]

    jmsDestType = self.packagePrefix + 'JMSDestinationRuntimeMBean'
    jmsDestAttributes = [
      'MessagesCurrentCount', 'MessagesPendingCount', 'ConsumersCurrentCount', 
      'BytesCurrentCount', 'BytesPendingCount'
      ]
    
    jmsPoolType = self.packagePrefix + 'JMSPooledConnectionRuntimeMBean'
    jmsPoolAttributes = ['AverageReserved','NumAvailable','NumLeaked','NumReserved','NumWaiters'] 
    jmsPoolWatch = {
      'JMSPoolWatch' : 
        { 'rule' : '${[' + jmsPoolType + ']//HealthState.State} != 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset' }
      }

    jmsEndPointType = 'JMSRemoteEndpointRuntimeMBean'
    jmsEndPointAttributes = [ 
      'BytesCurrentCount', 'BytesPendingCount', 'MessagesCurrentCount', 
      'MessagesPendingCount', 'MessagesReceivedCount'
      ]

    self.attributes = {
      jmsConnectionType : jmsConnectionAttributes,
      jmsDestType : jmsDestAttributes,
      jmsPoolType : jmsPoolAttributes,
      jmsEndPointType : jmsEndPointAttributes
      }

    if self.getWLSVersion() >= 10.3:
      self.watches = jmsPoolWatch

#  JTA Profile
#
class WLSJTAProfile(WLDFProfileBase):

  def __init__(self):
    
    jtaType = self.packagePrefix + 'JTARuntimeMBean'
    jtaAttributes = [
      'ActiveTransactionsTotalCount', 'TransactionAbandonedTotalCount', 
      'TransactionCommittedTotalCount', 'TransactionRolledBackTotalCount', 
      'TransactionTotalCount' ]
    jtaWatch = {
      'JTAWatch' : 
        { 'rule' : '${[' + jtaType + ']//HealthState.State} != 0', 'ruleType' : 'Harvester', 'resetType' : 'AutomaticReset' }
      }
    
    self.attributes = { jtaType : jtaAttributes }
    if self.getWLSVersion() >= 10.3:
      self.watches = jtaWatch
    
# WebApp Profile 
#
class WLSWebAppProfile(WLDFProfileBase):

  def __init__(self):
    
    webappType = self.packagePrefix + 'WebAppComponentRuntimeMBean'
    webappAttributes = ['OpenSessionsCurrentCount']
    
    servletType = self.packagePrefix + 'ServletRuntimeMBean'
    servletAttributes = ['ExecutionTimeAverage', 'ExecutionTimeHigh', 'ExecutionTimeLow', 'InvocationTotalCount']
     
    self.attributes = { 
      webappType  : webappAttributes,
      servletType : servletAttributes
      }
    self.watches = {}
    
# Web Services Profile
#
class WLSWebServicesProfile(WLDFProfileBase):

  def __init__(self):
    
    wseeType = self.packagePrefix + 'WseeOperationRuntimeMBean'
    wseeAttributes = ['DispatchTimeAverage','ExecutionTimeAverage','ResponseTimeAverage','ResponseErrorCount']
    wseeWatches = {
      'WSErrorWatch' : 
        { 'rule' : '${[' + wseeType + ']//ResponseErrorCount} > 0', 'ruleType' : 'Harvester', 'resetType' : 'ManualReset' }
      }
    
    self.attributes = {
      wseeType : wseeAttributes
      }
    self.watches = wseeWatches
    
# Class for applying and unapplying common WLDF profiles.  
#
# The general usage is to instantiate an instance of this class
# with the following parameters:
# - a WLDFResource instance (defined in 'wldfResource.py')
# - a dictionary of properties that control some of the options for the profile
#   such as targets for the WLDF resource, the set of notification names
#   to be applied to any Watch rules, and the reset period for Watch alarms
#   (see the "Arguments" section below)
# - a list of profile objects (see below)
# - an optional boolean indicator that controls whether or not the 
#   profile manager instance can open it's own edit session or not; this 
#   is to allow mutliple profile managers to be chained together in the
#   same script
# 
# A profile is defined as an object that supports the following operations:
#
# getHarvestedAttributes() - returns an dictionary of arrays of MBean attributes
#                            to be harvested, keyed by the MBean type
# getWatchRules() - returns an dictionaries Watch configuration properties, 
#                   where each dictionary key is a the Watch name and the
#                   value is a nested dictionary of Watch properties 
#                   (see Watch properties documentation below)
#
# Argument Keys:
# 
# "user"            Username used for connecting to server
# "pass"            Password used for connecting to server
# "url"             Connection URL
# "notifications"   Comma-separated list of WLDF notifications to use for watch rules 
# "resetPeriod"     Default reset period to use for automatic reset alarm types
# "harvesterPeriod" The sampling rate to set on the Harvester, in millis
# 
# Watch Property Keys:
#
# "rule"            The watch rule expression
# "ruleType"        The watch rule type ("Harvester", "Log", "Instrumentation")
# "resetType"       The reset type ("AutomaticReset", "ManualReset")
# "resetPeriod"     The reset period for a watch (in ms)
# 
# See WLDFProfileBase for more information on defining WLDF profiles that can be used
# by the WLDFProfileManager class
#
class WLDFProfileManager(EditFunctions):
  __args = None
  __resource = None
  __profiles = None
  
  def __init__(self, wldfResource, args, profiles, auto=false):
    EditFunctions.__init__(self,auto)
    self.__args = args
    self.__resource = wldfResource
    self.__profiles = profiles

  def __applyProfile(self, profile, notifs, resetPeriod):
    monitors = profile.getInstrumentationMonitors()
    print "monitors: "  + str(monitors)
    if monitors is not None and len(monitors) > 0:
      print "enabling instrumentation"
      self.__resource.getInstrumentation().setEnabled(true)
      self.__resource.enableDiagnosticContext(true)
      for monitorName, monProps  in monitors.items():
        print "Adding monitor " + monitorName
        monitor = self.__resource.findOrCreateInstrumentationMonitor(monitorName)
        monitor.setActions(monProps['actions'])
        monitor.setProperties(monProps['props'])
        monitor.setDescription(monProps['desc'])
        monitor.setDyeMask(monProps['dyeMask'])
        monitor.setDyeFilteringEnabled(monProps['dyeFiltering'])

    for typeName, attributeList in profile.getHarvestedAttributes().items():
      self.__resource.setAttributeList(typeName, attributeList)
    
    for watchName, watchProps in profile.getWatchRules().items():
        #print "Properties for watch " + watchName + ": " + str(watchProps)
        watchReset = getValue(watchProps, 'resetPeriod', resetPeriod)
        if watchReset is None:
          watchReset = 60000
        try:
          print "Creating watch " + watchName + "..."
          watch = self.__resource.addWatch(
            watchName=watchName, 
            watchRule=watchProps['rule'],
            watchType=watchProps['ruleType'], 
            watchAlarmType=watchProps['resetType'],
            watchResetPeriod=watchReset,
            watchNotifs=notifs)
        except BeanAlreadyExistsException:
          print "Watch " + watchName + " already exists"

  def __unapplyProfile(self, profile):
    #
    # Destroy all monitors
    #
    monitors = profile.getInstrumentationMonitors()
    for monitorName, monProps  in monitors.items():
      self.__resource.destroyInstrumentationMonitorByName(monitorName)
      
    #
    # Remove all Harvested types for this profile
    #
    harvester = self.__resource.getHarvester()
    allHarvestedTypes = harvester.getHarvestedTypes()
    harvestedAttributes = {}
    profileTypes = profile.getHarvestedAttributes() 
    for ht in allHarvestedTypes:
      # print "Checking harvested type in profile: " + ht.getName()
      if profileTypes.has_key(ht.getName()): 
        print "Destroying harvested type " +  ht.getName()
        harvester.destroyHarvestedType(ht)
    #
    # Remove Watches assocated with this profile
    #
    watchManager = self.__resource.getWatch()
    watches = watchManager.getWatches()
    profileWatches = profile.getWatchRules() 
    for watch in watches: 
      if profileWatches.has_key(watch.getName()): 
        print "Destroying watch " + watch.getName()
        watchManager.destroyWatch(watch)
   
  # returns the WLDFResource for this instance 
  def getWLDFResource(self):
    return self.__resource
  
  # Returns the arguments dictionary used by this instance
  def getArgs(self):
    return self.__args

  # Checks the connection state of the WLST shell and attempts
  # a connection if necessary.
  #
  def connectIfNecessary(self):
    connectIfNecessary(self.__args)
  
  # Apply the profiles managed by this instance
  #
  def apply(self):
    self.connectIfNecessary()
    try:
      currentDrive=currentTree()
      cd("edit:/")

      self.beginEdit()

      # find or create the SystemResource
      self.getWLDFResource().findOrCreateResource()
      # save the auto-edit state of the WLDFResource object and
      # turn it off for the duration of this method
      saveAutoEdit = self.getWLDFResource().isAutoEdit()
      self.getWLDFResource().setAutoEdit(false)
      
      try:
        # set the harvester period if provided
        harvesterPeriod = long(getValue(self.__args, 'harvesterPeriod', 0))
        if harvesterPeriod > 0:
          print "Setting Harvester sample period to " + str(harvesterPeriod) + " millis"
          self.getWLDFResource().getHarvester().setSamplePeriod(harvesterPeriod)
          
        # get the watch reset period, default to 2*Harvester period
        defaultPeriod = 2*self.getWLDFResource().getHarvester().getSamplePeriod()
        resetPeriod = long(getValue(self.__args, 'resetPeriod', defaultPeriod))
        
        notifs = None
        notifNames = getValue(self.__args, 'notifications', None)
        if notifNames is not None:
          print 'notifications: ' + notifNames
          notifNameList = notifNames.split(',')
          print 'notification list: ' + str(notifNameList)
          notifs = self.getWLDFResource().findNotifications(notifNameList)
        else:
          notifJMX = self.getWLDFResource().findOrCreateJMXNotification('myJMXNotif')
          notifSNMP = self.getWLDFResource().findOrCreateSNMPNotification('mySNMPNotif')
          notifs = [ notifJMX, notifSNMP ]
        
        for profile in self.__profiles:
          self.__applyProfile(profile, notifs, resetPeriod)
        
        targets = getValue(self.__args, 'targets', None)
        if targets is not None:
          targetsList = targets.split(",")
          self.getWLDFResource().targetResource(targetsList)
      
        self.activate()
      finally:
        self.getWLDFResource().setAutoEdit(saveAutoEdit)
        
    finally:
      self.cancelEdit()
      currentDrive()

  # Unapply the profiles managed by this instance
  def unapply(self):
    self.connectIfNecessary()
    self.getWLDFResource().findResource()
    saveAutoEdit = self.getWLDFResource().isAutoEdit()
    self.getWLDFResource().setAutoEdit(false)
    try:
      currentDrive=currentTree()
      cd("edit:/")

      self.beginEdit()

      for profile in self.__profiles:
        self.__unapplyProfile(profile)
      self.activate()

    finally:
      self.cancelEdit()
      self.getWLDFResource().setAutoEdit(saveAutoEdit)
      currentDrive()
  
