"""@package WLDFResource
Set of common utility functions and classes used by the other WLDF 
configuration profiles and examples.
 
This includes functionality for
- Retrieving positional arguments from the command line with a default value
- A class, WLDFResource, for managing the WLDFSystemResource bean and 
  performing common functions with it

To make use of these capabilities within WLST, simply run an execfile()
command with this script in the interpreter, or within your own script,
and make use of these utilities.

NOTE: This is intended for use within the WLST shell, both interactive and
in script mode.  It can not be used as-is if you are using WLST as a module
from Jython itself.  It can be made to do so by importing WLST as
module and prefixing each WLST command with the module prefix.

See http://dev2dev.bea.com/blog/sghattu/archive/2006/01/wlst_as_a_jytho.html
for a more complete explanation on this topic.
"""
from java.util import Date
from java.text import SimpleDateFormat
from java.io import ByteArrayOutputStream 
from java.io import StringBufferInputStream
from java.util import Properties
from java.lang import *
import jarray
import sys

# Function to handle script arguments of the variety 'n=v', where
# arguments are placed into a dictionary of nv pairs and returned
# to the caller
def argsToDict(args):
  d = {}
  for arg in args:
    #print "arg: " + arg
    pair = arg.split('=', 1)
    #print "pair: " + str(pair)
    if len(pair) == 2:
      # binary argument, store as key pair
      key = pair[0]
      val = pair[1]
      d[key] = val
    else:
      # Unary argument, story with empty (non-null) key
      d[arg] = ''
  print "Arguments: " + str(d)
  return d 
      
# Returns the value found in the provided map, at the location
# specified by 'key'; if no entry exists in the map for 'key',
# the provided default is returned. 
def getValue(dict, key, default=None):
  ret = default
  if dict is not None:
    try:
      ret=dict[key]
    except KeyError:
      pass
  return ret
  
# Connect to the target server specified in the provide args
def connectIfNecessary(argsDict=None):
  # connect if necessary
  if connected == "false":
    user=getValue(argsDict, "user", "weblogic")
    passwd=getValue(argsDict, "pass", "weblogic")
    url=getValue(argsDict, "url", "t3://localhost:7001")
    print "Connecting with [" + user + "," + passwd + "," + url + "]"
    connect(user,passwd,url)
      
# Retrieve a positional argument if it exists; if not,
# the provided default is returned.
#
# Params:
# pos   The integer location in sys.argv of the parameter
# default  The default value to return if the parameter does not exist
#
# returns the value at sys.argv[pos], or the provided default if necesssary
def getPositionalArgument(pos, default):
  value=None
  try:
    value=sys.argv[pos]
  except:
    value=default
  return value

def createTargetMBeanList(targetList):
    targetMBeans = []
    if targetList is not None:
      for targetName in targetList:
          print "Looking up target " + str(targetName)
          targetRef = cmo.lookupTarget(targetName)
          if targetRef != None:
              targetMBeans.append(targetRef)
              print "Adding target " + targetRef.getName() + " instance to list of targets..."
    return targetMBeans

# Base class for WLDFResource module classes; used for
# providing auto-edit capabilities.  If the auto-edit
# flag is enabled, the edit session can be controlled
# by the subclass as desired.  
#
# If disabled, the edit session is controlled externally
# to the subclass.
class EditFunctions:
  __autoEdit=false
  
  def __init__(self, auto=false):
    self.__autoEdit=auto
    if self.__autoEdit:
      print "Auto edit is enabled"
    else:
      print "Auto edit not enabled"
  
  def isAutoEdit(self):
    return self.__autoEdit
  
  def setAutoEdit(self, auto):
    self.__autoEdit = auto
    
  def cancelEdit(self):
    if self.isAutoEdit() and cmgr.isEditor():
      print "Canceling edit session..." 
      stopEdit("y")

  def activate(self):
    if self.isAutoEdit() and cmgr.isEditor():
      print "Activating changes..." 
      save()
      activate()

  def beginEdit(self):
    if self.isAutoEdit() and not cmgr.isEditor():
      print "Starting edit session..." 
      startEdit()

  
# Some common utils for performing related WLDF 
# configuration functions that are are not part 
# of the WLDFResourceBean configuration
#
# This includes enabling/disabling debug 
# attributes and scopes, creating SNMP agents,
# creating MailSessions, and so on.
#
class WLDFUtils(EditFunctions):
  
  def __init__(self, auto=false):
    EditFunctions.__init__(self, auto)
    
  # Enable a primitive debug attribute on the ServerDebugMBean
  def enableDebugAttribute(self, attributeName):
    self.setDebugAttribute(attributeName,true)
      
  # Disable a primitive debug attribute on the ServerDebugMBean
  def disableDebugAttribute(self, attributeName):
    self.setDebugAttribute(attributeName,false)
      
  # Enable a debug scope
  def enableDebugScope(self, scopeName):
    self.setDebugScope(scopeName, true)
      
  # Disable a debug scope
  def disableDebugScope(self, scopeName):
    self.setDebugScope(scopeName, false)
  
  # Set a value on a debug scope
  def setDebugScope(self, scopeName, value):
    currentDrive=currentTree()
    scope=None
    try:
      cd("edit:/Servers/"+serverName+"/ServerDebug/"+serverName)
      self.beginEdit() 
      scope=cmo.lookupDebugScope(scopeName)
      if scope == None:
          scope = cmo.createDebugScope(scopeName)
      scope.setEnabled(value)
      self.activate()
    finally:
      self.cancelEdit() 
      currentDrive()
    return scope
          
  # Set a value on a debug attribute
  def setDebugAttribute(self, attributeName, value):
    currentDrive=currentTree()
    try:
      cd("edit:/Servers/"+serverName+"/ServerDebug/"+serverName)
      self.beginEdit() #startEdit()
      print "type of value is " + str(type(value))
      set(attributeName,value)
      self.activate()
    finally:
      self.cancelEdit() 
      currentDrive()

  # Looks up a mail session or creates it if necessary
  def findOrCreateSMTPMailSession(self, sessionName, jndiName, sessionProperties, targets=None):
    currentDrive=currentTree()
    try:
      cd("edit:/")
      # Target the System Resource to the demo server
      print "looking up mail session " + sessionName
      mailSession=cmo.lookupMailSession(sessionName)
      if mailSession == None:
        self.beginEdit()
        print "Creating mail session " + sessionName + " with JNDI name " + jndiName
        print "Session properties: " + sessionProperties
        mailSession=cmo.createMailSession(sessionName)
        mailSession.setJNDIName(jndiName)
        mailSession.setProperties(makePropertiesObject(sessionProperties))
        if targets is not None:
          mailSession.setTargets(targets)
        self.activate()
    finally:
      self.cancelEdit()
      currentDrive()
    return mailSession
  
  ###########################################################################
  # Enable or disable the domain-wide SNMP agent, and create an SNMP
  # trap destination.  
  #
  # Params:
  # enable  Boolean that will enable or disable the agent (default is true)
  # trapDestName  The name of the trap destination configuration object, default is "mydest"
  # trapPort The port number for the trap destination, default is 4000
  #
  # return Returns the agent instance to the caller
  ###########################################################################
  def findOrCreateSNMPTrapDestination(self, trapDestName="mydest", trapPort=4000, enable=true):
    currentDrive=currentTree()
    dest = None
    try:
      cd ("edit:/")
      agent = enableDomainSNMPAgent(enable)
      dest = agent.lookupSNMPTrapDestination(trapDestName)
      if dest is None:
        self.beginEdit()
        dest = agent.createSNMPTrapDestination(trapDestName)
        dest.setPort(trapPort)
        self.activate()
    finally:
      self.cancelEdit()
      currentDrive()
    return dest
  
  ###########################################################################
  # Enable or disable the domain-wide SNMP agent.
  #
  # Params:
  # enable  Boolean that will enable or disable the agent (default is true)
  #
  # return Returns the agent instance to the caller
  ###########################################################################
  def enableDomainSNMPAgent(self):
    agent = None
    currentDrive=currentTree()
    try:
      cd ("edit:/")
      agent = cmo.getSNMPAgent()
      self.beginEdit()
      agent.setEnabled(enable)
      self.activate()
    finally:
      self.cancelEdit()
      currentDrive()
    return agent

# Utility class for performing common operations on
# the WLDFSystemResourceMBean and it's child bean 
# objects.
class WLDFResource(EditFunctions):

  __name = None
  systemResource = None
  
  # Constructor; takes a name of a WLDFSystemResource and
  # an auto-edit flag (default is false).  If connected,
  # the resource is looked up
  def __init__(self, resName, auto=false):
    EditFunctions.__init__(self,auto)
    self.__name = resName
    print "WLDFResource name: " + self.__name
    if connected == "true":
      print "connected, finding resource" + self.__name
      self.findResource()
    else:
      print "not connected"

  # Returns the child WLDFResourceBean instance for the
  # specified system resource.
  def getResource(self):
    if self.systemResource is not None:
      return self.systemResource.getWLDFResource()
    return None

  # Returns the WLDFHarvesterBean instance for the
  # specified system resource.
  def getHarvester(self):
    if self.systemResource is not None:
      return self.getResource().getHarvester()
    return None
  
  def getInstrumentation(self):
    if self.systemResource is not None:
      return self.getResource().getInstrumentation()
    return None
  
  # Returns the child WLDFWatchNotificationBean instance for the
  # specified system resource.
  def getWatch(self):
    if self.systemResource is not None:
      return self.getResource().getWatchNotification()
    return None

  # Looks up the WLDFSystemResourceMBean by name
  def findResource(self):
    currentDrive=currentTree()
    try:
      cd ("edit:/")
      self.systemResource = cmo.lookupWLDFSystemResource(self.__name)
    finally:
      currentDrive()
    return self.systemResource
  
  # Lookup a WLDFSystemResourceMBean by name in the WLST "edit" tree, and
  # create an instance for the specified name if one does not already exist.  If
  # specified, target to the specified Targetable entities.
  #
  # Params:
  # wldfResourceName    A string representing the name of the resource
  # target              Indicates whether or not to target the resource
  # targetList          A string array of the names of entities to target the
  #                     system resource to
  #
  # return the WLDFSystemResourceMBean instance for the specified name.
  def findOrCreateResource(self, targetList=None):
    currentDrive=currentTree()
    try:
      cd ("edit:/")
      self.systemResource = cmo.lookupWLDFSystemResource(self.__name)
      if self.systemResource is None:
        self.beginEdit()
        print "Creating new WLDF System resource for " + str(self.__name)
        self.systemResource=cmo.createWLDFSystemResource(self.__name)
      # Check if targeting is requested
      if targetList is not None:
        print "Targeting resource..."
        self.beginEdit()
        self.targetResource(targetList)
      self.activate()
    finally:
      self.cancelEdit()
      currentDrive()
    return self.systemResource
  
  # Target the SystemResourceMBean specified to the list of
  # TargetMBeans provided.  Works for all SystemResourceMBeans.
  #
  # Params:
  # systemResource   A SystemResourceMBean instance
  # targetList       A list of TargetMBeans to target
  def targetResource(self, targetList = None):
    print "Targets: " + str(targetList)
    currentDrive=currentTree()
    try:
      cd("edit:/")
      targetMBeans = createTargetMBeanList(targetList)
      print "Setting targets on " + self.systemResource.getName()
      self.systemResource.setTargets(targetMBeans)
    finally:
      currentDrive()
      
  # Remove all targets from the named SystemResourceMBean; not specific to 
  # WLDF system resource MBeans.
  # 
  # Params:
  # systemResourceName  The name of the SystemResource
  def untargetAll(self):
    currentDrive=currentTree()
    try:
      cd ("edit:/")
      self.systemResource = cmo.lookupWLDFSystemResource(self.__name)
      if self.systemResource != None:
        self.beginEdit()
        targets = self.systemResource.getTargets()
        for target in targets:
          print "Removing target " + target.getName() + " from " + self.systemResource.getName()
          self.systemResource.removeTarget(target)
        self.activate()
    finally:
      self.cancelEdit()
      currentDrive()
        
  # Destroy the named WLDF system resource, untargeting it if necessary
  # first
  #
  # Params:
  # systemResourceName  The name of the SystemResource
  def destroyResource(self):
    currentDrive=currentTree()
    try:
      cd("edit:/")
      if self.systemResource is not None:
        self.beginEdit()
        cmo.destroyWLDFSystemResource(self.systemResource)
        self.activate()
        self.systemResource=None
    finally:
      self.cancelEdit()
      currentDrive()
        
  ##############################################################################
  # Helper functions for adding types/attributes to the harvester configuration
  ##############################################################################
  
  # Find if the specified type name exists in the harvester configuration; 
  # return it if found, None if not.
  #
  # Params:
  # harvester      The WLDFHarvesterBean representing the harvester configuration
  # typeName       A string identifying the MBean type; this can be either
  #                the fully qualified type name for WLS Runtime MBeans, or
  #                the name specified in the ModelMBeanInfo for a Custom MBean type
  #
  # return The WLDFHarvestedTypeBean instance representing the harvester
  # configuration for the type; "None" if it is not found in the harvester 
  # configuration provided.
  def findHarvestedType(self, typeName):
    harvester = self.getHarvester()
    htypes=harvester.getHarvestedTypes()
    for ht in (htypes):
      if ht.getName() == typeName:
        return ht
    return None
  
  def findOrCreateHarvestedType(self, typeName, knownType=0):
    ht = self.findHarvestedType(typeName)
    if ht is None:
      ht = self.addTypeByName(typeName, knownType)
    return ht

  # Find if the specified monitor name exists in the Instrumentation configuration; 
  # return it if found, None if not.
  #
  # Params:
  # instBean     The WLDFInstrumentationBean representing the server's instrumentation configuration
  # monitorName  The monitor name to look for
  def findInstrumentationMonitor(self, monitorName):
    monitors=self.getInstrumentation().getWLDFInstrumentationMonitors()
    for m in monitors:
      if m.getName() == monitorName:
        return m
    return None
  
  # Find if the specified monitor name exists in the Instrumentation configuration; 
  # return it if found, None if not.
  #
  # Params:
  # instBean          The WLDFInstrumentationBean representing the server's instrumentation configuration
  # monitorName  The monitor name to look for
  # actions          The actions to attach the monitor, as an array of Strings (default is None)
  # props            A java.util.Properties object that will be set as a String on the monitor  (default is None)
  # dyeMask       The dye mask to trigger the monitor on, if dye filtering is enabled (default is None)
  # dyeFiltering    Enable or disable dye filtering (false by default)
  # locationType  A string identifying the type of pointcut ("before", "after", "around") 
  # pointCut        The pointcut expression, for Custom monitors only  (default is None)
  # includes         The set of classes to scan to apply this monitor (default is None)
  # excludes        The set of classes to excludes from the weaving process for this monitor (default is None) 
  # enabled       Whether or not the monitor is enabled  (default is true)
  # description  Description of the monitor (default is None)
  def findOrCreateInstrumentationMonitor(self, monitorName,
      actions=None, props=None, dyeMask=None, dyeFiltering=false, 
       locationType=None, pointcut=None, includes=None, excludes=None, 
      enabled=true, description=None):
    monitor = self.findInstrumentationMonitor(monitorName)
    if monitor is None:
      currentDrive=currentTree()
      try:
        cd("edit:/")
        self.beginEdit()
        monitor = self.getInstrumentation().createWLDFInstrumentationMonitor(monitorName)
        monitor.setEnabled(enabled)
        monitor.setDyeFilteringEnabled(dyeFiltering)
        if actions is not None:
          monitor.setActions(actions)
        if props is not None:
          self.addInstrumentationMonitorProperties(monitor, props)
        if locationType is not None:
          monitor.setLocationType(locationType)
        if pointcut is not None:
          monitor.setPointcut(pointcut)
        if includes is not None:
          monitor.setIncludes(includes)
        if excludes is not None:
          monitor.setExcludes(excludes)
        if description is not None:
          monitor.setDescription(description)
        self.activate()
      finally:
        self.cancelEdit()
        currentDrive()
    return monitor
  
  # Destroy an instrumentation monitor by name
  # Parameters:
  # monitorName  The name of the monitor
  #
  def destroyInstrumentationMonitorByName(self, monitorName):
    self.destroyInstrumentationMonitor(self.findInstrumentationMonitor(monitorName))
    
  # Destroy an instrumentation monitor bean
  # Parameters:
  # monitor   The monitor bean instance
  #
  def destroyInstrumentationMonitor(self, monitor):
    if monitor is not None:
      self.getInstrumentation().destroyWLDFInstrumentationMonitor(monitor)
  
  def loadPropertiesString(self, propsString):
    p = Properties()
    if propsString is not None:
      p.load(StringBufferInputStream(propsString))
    return p
  
  def addMonitorPropertiesString(self, monitor, propsString):
    print "propsString: " + propsString
    props = self.loadPropertiesString(propsString)
    self.addMonitorProperties(monitor, props)

  # Adds a set of properties to an instrumentation monitor's existing properties
  #
  # monitor  The instrumentation monitor bean
  # props     The java.util.Properties object containing the properties 
  def addMonitorProperties(self, monitor, props):
    if props is not None:
      currentDrive=currentTree()
      try:
        cd("edit:/")
        self.beginEdit()
        newProps = self.loadPropertiesString(monitor.getProperties())
        newProps.putAll(props)
        print "newProps" + str(newProps)
        w=ByteArrayOutputStream()
        newProps.store(w, "Dye Properties for monitor "  + monitor.getName())
        monitor.setProperties(w.toString())
        self.activate()
      finally:
        self.cancelEdit()
        currentDrive()

  # Removes a set of properties from an instrumentation monitor's existing properties
  #
  # monitor  The instrumentation monitor bean
  # props     The java.util.Properties object containing the properties 
  def removeMonitorProperties(self,  monitor, props):
    if props is not None:
      currentDrive=currentTree()
      try:
        cd("edit:/")
        self.beginEdit()
        monProps = Properties()
        monProps = self.loadPropertiesString(monitor.getProperties())
        for k in props.keySet():
          monProps.remove(k)
        w=ByteArrayOutputStream()
        monProps.store(w, "Dye Properties for monitor "  + monitor.getName())
        monitor.setProperties(w.toString())
        self.activate()
      finally:
        self.cancelEdit()
        currentDrive()
       
  # Toggles the Diagnostic context functionality
  def enableDiagnosticContext(self, enable=true):
    currentDrive=currentTree()
    try:
      cd("edit:/")
      self.beginEdit()
      # enable DC tracking
      diagnosticConfig=getMBean("edit:/Servers/" + serverName + "/ServerDiagnosticConfig/" + serverName)
      diagnosticConfig.setDiagnosticContextEnabled(enable)
    finally:
      self.cancelEdit()
      currentDrive()
    
  # Creates a DyeInjection monitor and enables the DiagnosticContext functionality
  def enableDyeInjection(self, dyeprops=None, enable=true):
    currentDrive=currentTree()
    try:
      cd("edit:/")
      self.beginEdit()
      self.getInstrumentation().setEnabled(enable)
      dim = self.findOrCreateInstrumentationMonitor("DyeInjection", props=dyeprops)
      self.setAutoEdit(false)
      self.enableDiagnosticContext(enable)
      self.setAutoEdit(true)
      self.activate()
    finally:
      self.cancelEdit()
      currentDrive()
    return monitor

  # Sets a list of attributes on a harvested type, creating
  # the WLDFHarvestedTypeBean instance if necessary
  #
  # Params:
  # typeName       A string identifying the MBean type; this can be either
  #                the fully qualified type name for WLS Runtime MBeans, or
  #                the name specified in the ModelMBeanInfo for a Custom MBean type
  # attributeName  The names of the attributes to add to the collection
  #
  def setAttributeList(self, typeName, attributes, knownType=0):
    ht = self.findOrCreateHarvestedType(typeName, knownType)
    if ht is None:
      ht = self.addTypeByName(fullType, true)
    ht.setHarvestedAttributes(jarray.array(attributes, java.lang.String))
    
  # Builds a type-name and adds it to the harvester configuration, 
  # assuming that the mbeanInstance is a WLS RuntimeMBean; returns the
  # WLDFHarvestedTypeBean representing the type name in the harvester 
  # configuration.
  #
  # Params:
  # mbeanInstance  A WLS RuntimeMBean instance 
  #
  # return An instance of WLDFHarvestedTypeBean representing the harvester
  # configuration for the type
  def addTypeForInstance(self, mbeanInstance):
    typeName = "weblogic.management.runtime." + mbeanInstance.getType() + "MBean"
    return addTypeByName(typeName, 1)
  
  # Find the harvested type in the harvester configuration provided, 
  # and create one if it is not found.  Returns an instance of the
  # WLDFHarvestedTypeBean for the type name.
  #
  # Params:
  # typeName       A string identifying the MBean type; this can be either
  #                the fully qualified type name for WLS Runtime MBeans, or
  #                the name specified in the ModelMBeanInfo for a Custom MBean type
  # knownType      Indicates if this type is a known WLS MBean type
  #
  # return An instance of WLDFHarvestedTypeBean representing the harvester
  # configuration for the type
  def addTypeByName(self, _typeName, knownType=0):
    ht=self.findHarvestedType(_typeName)
    if ht == None:
      print "Adding " + _typeName + " to harvestables collection for " + self.getResource().getName()
      ht=self.getHarvester().createHarvestedType(_typeName)
      if knownType == 1:
          print "Setting known type attribute to true for " + _typeName
          ht.setKnownType(knownType)
    return ht;
  
  # Adds an attribute name to the havested type, if it has not already
  # been added previously.
  #
  # Params:
  # harvestedType  The WLDFHarvestedTypeBean to modify
  # attributeName  The name of the attribute to add to the collection
  #
  # return The passed-in WLDFHarvestedTypeBean 
  def addAttributeToHarvestedType(self, harvestedType, targetAttribute):
    # create a temporary PyList to hold the existing attributes
    # and then convert the new list back to a Java array using
    # the jarray utility package.
    currentAttributes = PyList()
    currentAttributes.extend(harvestedType.getHarvestedAttributes());
    print "Current attributes: " + str(currentAttributes)
    try:
      currentAttributes.index(targetAttribute)
      print "Attribute is already in set"
      return
    except ValueError:
      print targetAttribute + " not in list, adding"
        
    currentAttributes.append(targetAttribute)
    newSet = jarray.array(currentAttributes, java.lang.String)
    print "New attributes for type " + harvestedType.getName() + ": " + str(newSet)
    
    harvestedType.setHarvestedAttributes(newSet)
    return harvestedType
  
  # Adds an attribute name to the named type configuration; the 
  # harvested type bean is created if necessary.
  #
  # Params:
  # harvester      The WLDFHarvesterBean representing the harvester configuration
  # typeName       A string identifying the MBean type; this can be either
  #                the fully qualified type name for WLS Runtime MBeans, or
  #                the name specified in the ModelMBeanInfo for a Custom MBean type
  # attributeName  The name of the attribute to add to the collection
  # knownType      Indicates if this type is a known WLS MBean type
  #
  # return An instance of WLDFHarvestedTypeBean representing the harvester
  # configuration for the type
  def addAttributeForTypeByName(self, typeName, attributeName, knownType=1):
    ht = addTypeByName(typeName, knownType)
    return addAttributeToHarvestedType(ht,attributeName)
  
  # Adds an attribute name to the type configuration represented by
  # the MBean instance argument.  The attribute type is added to
  # the harvester configuration if necessary.
  #
  # Params:
  # harvester      The WLDFHarvesterBean representing the harvester configuration
  # mbeanInstance  A WLS RuntimeMBean instance 
  # attributeName  The name of the attribute to add to the collection
  #
  # return An instance of WLDFHarvestedTypeBean representing the harvester
  # configuration for the type
  def addAttributeForTypeByInstance(self, mbeanInstance, attributeName):
    ht = addTypeForInstance(mbeanInstance, 1)
    return addAttributeToHarvestedType(ht,attributeName)
  
  # Adds an attribute name to the instance-based configuration 
  # represented by the MBean instance argument.  The attribute 
  # type is added to the harvester configuration if necessary.
  #
  # Params:
  # harvester      The WLDFHarvesterBean representing the harvester configuration
  # mbeanInstance  A WLS RuntimeMBean instance 
  # attributeName  The name of the attribute to add to the collection
  #
  # return An instance of WLDFHarvestedTypeBean representing the harvester
  # configuration for the type
  def addAttributeForInstance(self, mbeanInstance, attributeName):
    typeName = mbeanInstance.getType() + "MBean"
    ht = addInstanceToHarvestedType(mbeanInstance)
    return addAttributeToHarvestedType(ht,attributeName)
  
  # Adds an MBean instance to its harvested type specification in
  # the harvester configuration provided.  
  #
  # Params:
  # harvester      The WLDFHarvesterBean representing the harvester configuration
  # mbeanInstance  A WLS RuntimeMBean instance 
  #
  # return An instance of WLDFHarvestedTypeBean representing the harvester
  # configuration for the type
  def addInstanceToHarvestedType(self, mbeanInstance):
    # Find or create the WLDFHarvestedTypeBean representing the 
    # type of the mbeanInstance specified
    harvestedType = addTypeForInstance(mbeanInstance)
    currentInstances = PyList()  # temp PyList to search for the ObjectName in the list
    currentInstances.extend(harvestedType.getHarvestedAttributes());
    on = mbeanInstance.getObjectName().getCanonicalName()
    print "Adding " + str(on) + " to set of harvested instances for type " \
          + harvestedType.getName()
    print "Current instances : " + str(currentInstances)
    for inst in currentInstances:
      if inst == on:
        print "Found " + on + " in existing set"
        return harvestedType
    # only get here if the target attribute is not in the set
    currentInstances.append(on)
    # convert the new list back to a Java String array 
    newSet = jarray.array(currentInstances, java.lang.String)
    print "New instance set for type " + harvestedType.getName() + ": " + str(newSet)
    harvestedType.setHarvestedInstances(newSet)
    return harvestedType
  
  # Reset the harvester configuration (destroy all configured
  # harvested types) for an existing WLDFSystemResourceMBean.
  #
  # Params:
  # wldfResourceName   The WLDF system resource name
  #
  # return the WLDFSystemResourceMBean for the name
  #         ("None" if it doesn't exist)
  def resetHarvester(self):
    currentDrive=currentTree()
    try:
      print "Resetting harvester for WLDF system resource " + self.getResource().getName()
      cd ("edit:/")
      if systemResource is None:
        systemResource = cmo.lookupWLDFSystemResource(self.__name)
      if wldfRes != None:
        print "Found WLDF system resource " + wldfRes.getName()
        startEdit()
        harvester = wldfRes.getWLDFResource().getHarvester()
        allHarvestedTypes = harvester.getHarvestedTypes()
        for ht in allHarvestedTypes:
            print "Destroying harvested type " +  ht.getName()
            harvester.destroyHarvestedType(ht)
        activate()
    finally:
      if cmgr.isEditor():
        stopEdit("y")
      currentDrive()
    return wldfRes
      
  # Destroy all watches attached to the provided WLDFWatchNotificationBean.
  #
  # Params:
  # watchNotification  The WLDFWatchNotificationBean representing the 
  #                    watch configuration
  #
  # return the provided WLDFWatchNotificationBean
  def resetWatches(self, watchNotification):
    if watchNotification == None:
        return
    watches = watchNotification.getWatches()
    for watch in watches:
        print "Removing watch " + watch.getName()
        watchNotification.destroyWatch(watch)
    return watchNotification
  
  # Destroy all notifications attached to the provided WLDFWatchNotificationBean.
  #
  # Params:
  # watchNotification  The WLDFWatchNotificationBean representing the 
  #                    watch configuration
  #
  # return the provided WLDFWatchNotificationBean
  def resetNotifications(self):
    watchNotification = self.getWatch()
    if watchNotification == None:
        return
    
    notifs = watchNotification.getJMSNotifications()
    for n in notifs:
        print "Removing JMS notification " + n.getName()
        watchNotification.destroyJMSNotification(n)
        
    notifs = watchNotification.getImageNotifications()
    for n in notifs:
        print "Removing Image notification " + n.getName()
        watchNotification.destroyImageNotification(n)
    
    notifs = watchNotification.getJMXNotifications()
    for n in notifs:
        print "Removing JMX notification " + n.getName()
        watchNotification.destroyJMXNotification(n)
    
    notifs = watchNotification.getSMTPNotifications()
    for n in notifs:
        print "Removing SMTP notification " + n.getName()
        watchNotification.destroySMTPNotification(n)
    
    notifs = watchNotification.getSNMPNotifications()
    for n in notifs:
        print "Removing SNMP notification " + n.getName()
        watchNotification.destroySNMPNotification(n)
    return watchNotification
  
  # Reset the entire watch and notifications settings on 
  # an existing WLDFSystemResourceMBean.
  #
  # Params:
  # wldfResourceName   The WLDF system resource name
  #
  # return the WLDFSystemResourceMBean for the name
  #         ("None" if it doesn't exist)
  def resetWatchAndNotifications(self, wldfResourceName):
    currentDrive=currentTree()
    try:
      cd ("edit:/")
      wldfRes = cmo.lookupWLDFSystemResource(wldfResourceName)
      if wldfRes == None:
        startEdit()
        watchbean = wldfRes.getWLDFResource().getWatchNotification()
        destroyAllWatches(watchbean)
        destroyAllNotifications(watchbean)
        activate()
    finally:
      if cmgr.isEditor():
        stopEdit("y")
      currentDrive()
    return wldfRes
      
  ###########################################################################
  # Display the currently registered types for the specified harvester
  #
  # Params:
  # harvester  The WLDFHarvesterBean representing the harvester configuration
  ###########################################################################
  def displayHarvestedTypes(self):
    harvestedTypes = self.getHarvester().getHarvestedTypes()
    print ""
    print "Harvested types:"
    print ""
    for ht in (harvestedTypes):
      print "Type: " + ht.getName()
      attributes = ht.getHarvestedAttributes()
      if attributes != None:
        print "  Attributes: " + str(attributes)
      instances = ht.getHarvestedInstances()
      print "  Instances:  " + str(instances)
      print ""
    return
  
  def displayWatches(self):
    watches = self.getWatch().getWatches()
    print ""
    print "Watches:"
    print ""
    for watch in watches:
      print "Watch: " + watch.getName()
      print "  Type:    " + watch.getRuleType()
      print "  Rule:    " + watch.getRuleExpression()
      print "  Alarm:   " + watch.getAlarmType()
      print "  Reset:   " + str(watch.getAlarmResetPeriod())
      print "  Enabled: " + str(watch.isEnabled())
      print "  Notifs:  " + str(watch.getNotifications())
    return
  
  def dumpResource(self):
    self.displayHarvestedTypes()
    print ""
    self.displayWatches()
    print ""
    return
  
  def addWatch(self, watchName, watchAlarmType, watchResetPeriod, watchRule, watchType="Harvester", watchNotifs = None):
    watchManager = self.getWatch()
    print "Adding watch '" + watchName + "', type '" + watchType + "', with rule '" +  watchRule + "', reset type: '" + watchAlarmType + "', reset Period: " + str(watchResetPeriod)
    watch = watchManager.createWatch(watchName)
    watch.setRuleType(watchType)
    watch.setRuleExpression(watchRule)
    watch.setAlarmType(watchAlarmType)
    watch.setAlarmResetPeriod(watchResetPeriod)
    watch.setEnabled(1)
    print "notifs: " + str(watchNotifs)
    if watchNotifs is not None:
      notifsArray = jarray.array(watchNotifs, weblogic.diagnostics.descriptor.WLDFNotificationBean)
      watch.setNotifications(notifsArray)
    return watch

  # Find the WLDFNotificationBean instances associated with the provided 
  # list of names; a list of those instances is returned to the caller.
  #
  # Params:
  # nameList  A list of notification names
  #
  # return The list of WLDFNotificationBean instances found 
  def findNotifications(self, nameList):
    notifs = []
    watch = self.getWatch()
    for name in nameList:
      n = watch.lookupNotification(name)
      if n is not None:
        #print "Found notif ication" + n.getName()
        notifs.append(n)
    #print "Num notifs found: "  + str(len(notifs))
    return notifs
  
  # Find the WLDFJMXNotificationBean instances associated with the 
  # specified name; if one does not exist it will be created
  #
  # Params:
  # nameList  A list of notification names
  #
  # return The requested WLDFJMXNotificationBean instance 
  def findOrCreateJMXNotification(self, name, notifType=None):
    watch = self.getWatch()
    n = watch.lookupNotification(name)
    if n is None:
      try:
        self.beginEdit()
        n = watch.createJMXNotification(name)
        if notifType is not None:
          n.setNotificationType(notifType)
        self.activate()
      finally:
        self.cancelEdit()
    return n
  
  # Find the WLDFJMXNotificationBean instances associated with the 
  # specified name; if one does not exist it will be created
  #
  # Params:
  # nameList  A list of notification names
  #
  # return The requested WLDFJMXNotificationBean instance 
  def findOrCreateSNMPNotification(self, name):
    watch = self.getWatch()
    n = watch.lookupNotification(name)
    if n is None:
      try:
        self.beginEdit()
        n = watch.createSNMPNotification(name)
        self.activate()
      finally:
        self.cancelEdit()
    return n
  
  def findOrCreateSMTPNotification(self, name,jndiName=None,recipients=None,subject=None,body=None):
    watch = self.getWatch()
    n = watch.lookupNotification(name)
    if n is None:
      try:
        self.beginEdit()
        n = watch.createSMTPNotification(name)
        if jndiName is not None:
          n.setMailSessionJNDIName(jndiName)
        if recipients is not None:
          n.setRecipients(recipients)
        if subject is not None:
          n.setSubject(subject)
        if body is not None:
          n.setBody(body)
        self.activate()
      finally:
        self.cancelEdit()
    return n

  def findNotificationsFromList(self, names):
    notifList = []
    if names is not None:
      nameList = names.split(',')
      notifList = self.findNotifications(nameList)
    return notifList
      
  def destroySMTPNotifications(self, names):
    notifList = self.findNotificationsFromList(names)
    try:
      self.beginEdit()
      for n in notifList:
        self.getWatch().destroySMTPNotification(n)
      self.activate()
    finally:
      self.cancelEdit()
      
  def destroySNMPNotifications(self, names):
    notifList = self.findNotificationsFromList(names)
    try:
      self.beginEdit()
      for n in notifList:
        self.getWatch().destroySNMPNotification(n)
      self.activate()
    finally:
      self.cancelEdit()

  def destroyJMXNotifications(self, names):
    notifList = self.findNotificationsFromList(names)
    try:
      self.beginEdit()
      for n in notifList:
        self.getWatch().destroyJMXNotification(n)
      self.activate()
    finally:
      self.cancelEdit()
      
  def destroyJMSNotifications(self, names):
    notifList = self.findNotificationsFromList(names)
    try:
      self.beginEdit()
      for n in notifList:
        self.getWatch().destroyJMSNotification(n)
      self.activate()
    finally:
      self.cancelEdit()
      
# Helper class for querying for harvested data and displaying
# the formatted data records.
class Accessor:
  
  __accessor = None
  __args = None
  __columnInfo = None
  __fetchMax = 100
  
  def __init__(self, type='ServerLog', args=None, fetchMax=100):
    connectIfNecessary(args)
    print "Looking up accessor " + type + " in serverRuntime tree..."
    accessRuntime = getMBean("serverRuntime:/WLDFRuntime/WLDFRuntime/WLDFAccessRuntime/Accessor")
    self.__accessor = accessRuntime.lookupDataAccessRuntime(type)
    self.__columnInfo = self.__accessor.getColumns()
    self.__fetchMax = 100
    
  def getAccessor(self):
    return self.__accessor
  
  # Query over the specified interval and return the resultant rows
  #
  # Params:
  #  begin Interval start time (millis), default is 0
  #  end   Interval end time (millis), default is Long.MAX_VALUE
  #  queryString The query string to execute (default is "", or all records)
  #  timeout  The timeout period for the cursor operation
  #  display  Whether or not to display the records to the console (false by default)
  def query(self, queryString="", begin=0, end=Long.MAX_VALUE, timeout=300000, displayOnly=false):
    result = []
    cursorName=self.__accessor.openCursor(begin, end, queryString, timeout)
    while self.__accessor.hasMoreData(cursorName):
      rows = self.__accessor.fetch(cursorName, self.__fetchMax)
      result.extend(rows)
      if displayOnly:
        self.__printItWithColumns(rows)
    self.__accessor.closeCursor(cursorName)
    print "TotalRecords: " + str(len(result))
    if not displayOnly:
      return result
    return None
  
  # Find most recent records, from the current time back over the specified
  # time interval in millis (default is 5 minutes)
  #
  # Params:
  #  interval  Time interval in millis (default is 300000)
  #  queryString The query string to execute (default is "", or all records)
  #  timeout  The timeout period for the cursor operation
  #  display  Whether or not to display the records to the console (false by default)
  def queryMostRecent(self, queryString="", interval=300000, timeout=300000, display=false):
    end=System.currentTimeMillis()
    start = end - interval
    return self.query(queryString, start, end, timeout, display)
  
  def __printDataRecordWithColumns(self, record):
    datePattern = "MM/dd/yyyy HH:mm:ss.SSS"
    formatter = SimpleDateFormat(datePattern)
    for col in range(0,len(record),1):
      val = record[col]
      colname = self.__columnInfo[col].getColumnName()
      print "  [" + str(col) + "] " + colname + ": " + str(val)
        
  def __printItWithColumns(self, rows):
    recno = 0
    for record in rows:
      recno += 1;
      print ""
      print "Record " + str(recno) + ":"
      self.__printDataRecordWithColumns(record)
      print ""
  
